(* ------------------------------------------------------ *)
(*                     LBM.MOD                            *)
(*          Implementationsmodul fr LBM                  *)
(*    Das Modul lbm stellt Routinen zum Einlesen von      *)
(*    IFF-Bildern und Farbpaletten in den VGA-Modi 18     *)
(*              und 19 zur Verfgung.                     *)
(*                                                        *)
(*        (C) 1992 Dietmar Ring & DMV-Verlag              *)
(*             Compiler: TopSpeed Modula                  *)
(* ------------------------------------------------------ *)


IMPLEMENTATION MODULE lbm;

FROM  FIO      IMPORT  RdBin,Seek,Open,Close;
FROM  Lib      IMPORT  WordMove,Intr,IncAddr,Move,Dos,
                       WordFill,Delay,Fill,AddAddr;
FROM  SYSTEM   IMPORT  Out,In,Registers;
FROM  Graph    IMPORT  SetVideoMode;
FROM  MATHLIB  IMPORT  Pow;
FROM  Storage  IMPORT  HeapRecPtr,MainHeap,
                       HeapAvail,HeapAllocate,
                       HeapDeallocate,MakeHeap,
                       ALLOCATE,DEALLOCATE,
                       Available;

(********************************************)

(* Prozeduren die die Daten aus dem Kopf der *)
(* IFF Datei ins Intel-Format umwandeln      *)

PROCEDURE cswap (VAR x:CARDINAL);
TYPE arr = ARRAY [1..2] OF BYTE;
VAR  a : arr;
     b : BYTE;
BEGIN
 WordMove(ADR(x),ADR(a),1);
 b:=a[2];
 a[2]:=a[1];
 a[1]:=b;
 WordMove(ADR(a),ADR(x),1);
END cswap;

(* ---------------------------------- *)

PROCEDURE iswap (VAR x:INTEGER);
TYPE arr = ARRAY [1..2] OF BYTE;
VAR  a : arr;
     b : BYTE;
BEGIN
 WordMove(ADR(x),ADR(a),1);
 b:=a[2];
 a[2]:=a[1];
 a[1]:=b;
 WordMove(ADR(a),ADR(x),1);
END iswap;

(* ---------------------------------- *)

PROCEDURE lswap (VAR x:LONGCARD);
VAR  a  : ARRAY [1..4] OF BYTE;
     b  : ARRAY [1..4] OF BYTE;
BEGIN
 WordMove(ADR(x),ADR(a),2);
 b[4]:=a[1];
 b[3]:=a[2];
 b[2]:=a[3];
 b[1]:=a[4];
 WordMove(ADR(b),ADR(x),2);
END lswap;

(********************************************)

(* Whlt eine Bit-Plane der VGA-Karte aus *)

PROCEDURE wplane(n:CARDINAL);
BEGIN
 CASE n OF
  1: Out (964,2);
     Out (965,1);|
  2: Out (964,2);
     Out (965,2);|
  3: Out (964,2);
     Out (965,4);|
  4: Out (964,2);
     Out (965,8);
 END;
END wplane;

(********************************************)

PROCEDURE Wait();
VAR count : CARDINAL;
    wert  : BYTE;
BEGIN
 FOR count := 1 TO 2 DO
  REPEAT
  UNTIL (3 IN BITSET(In(03DAH)));
 END;
END Wait;

(* ---------------------------------- *)

(* Off und On schalten die       *)
(* Bildschirmausgabe ein bzw aus *)

PROCEDURE Off();
VAR   wert  : BYTE;
BEGIN
 Wait();
 Out(03C4H,1);
 wert := In(03C5H);
 INCL(BITSET(wert),5);
 Out(03C5H,wert);
END Off;

PROCEDURE On();
VAR   wert  : BYTE;
BEGIN
 Wait();
 Out(03C4H,1);
 wert := In(03C5H);
 EXCL(BITSET(wert),5);
 Out(03C5H,wert);
END On;

(********************************************)

PROCEDURE IstLbm(datei : File):BOOLEAN;
TYPE   string4    = ARRAY[1..4] OF CHAR;
VAR    datkennung : string4;
       kennung    : string4;
       result     : CARDINAL;
BEGIN
 kennung := 'FORM';
 Seek(datei,0);
 result := RdBin(datei,datkennung,4);
 IF kennung=datkennung THEN
  RETURN(TRUE);
 ELSE
  RETURN(FALSE);
 END;
END IstLbm;

(********************************************)

PROCEDURE MakeVirt16():virt16_typ;
VAR count    :  CARDINAL;
    count2   :  CARDINAL;
    virt16   :  virt16_typ;
BEGIN
 FOR count := 1 TO 4 DO
  virt16[count] := NIL;
  IF Available(SIZE(screen16_typ)) THEN
   ALLOCATE(virt16[count],SIZE(screen16_typ));
  ELSE
   FOR count2 := 1 TO 4 DO
    IF virt16[count2]<>NIL THEN
     DEALLOCATE(virt16[count2],SIZE(screen16_typ));
     virt16[count2] := NIL;
    END;
   END;
   RETURN (virt16);
  END;
 END;
 RETURN(virt16);
END MakeVirt16;

(* ---------------------------------- *)

PROCEDURE RemoveVirt16(virt16 : virt16_typ);
VAR  count : CARDINAL;
BEGIN
 FOR count := 1 TO 4 DO
  IF virt16[count]<>NIL THEN
   DEALLOCATE(virt16[count],SIZE(screen16_typ));
  END;
 END;
END RemoveVirt16;

(* ---------------------------------- *)

PROCEDURE MakeVirt256():virt256_pointer_typ;
VAR virt256   :  virt256_pointer_typ;
BEGIN
 virt256 := NIL;
 IF Available(SIZE(screen256_typ)) THEN
  ALLOCATE(virt256,SIZE(screen256_typ));
  WordFill(virt256,SIZE(screen256_typ) DIV 2,0);
 END;
 RETURN(virt256);
END MakeVirt256;

(* ---------------------------------- *)

PROCEDURE RemoveVirt256(virt256 : virt256_pointer_typ);
BEGIN
  IF virt256<>NIL THEN
   DEALLOCATE(virt256,SIZE(screen256_typ));
  END;
END RemoveVirt256;

(* ---------------------------------- *)

PROCEDURE ToScreen16(quelle : virt16_typ;
                     mode : CARDINAL );
VAR   count  : CARDINAL;
      count2 : CARDINAL;
      count3 : CARDINAL;
      plane  : CARDINAL;
BEGIN
 CASE mode OF
  1    :   Off();
           FOR plane := 1 TO 4 DO
            wplane(plane);
            WordMove(quelle[plane],ADR(screen1[1,1]),19200);
           END;
           On();|
  2    :  FOR count := 1 TO 40 DO
           FOR count2 := 1 TO 480 DO
            FOR plane := 1 TO 4 DO
             wplane(plane);
             screen1[count2,40 + count]
             := quelle[plane]^[count2,40 + count];
             screen1[count2,41 - count]
             := quelle[plane]^[count2,41 - count];
            END;
           END;
           Delay(speedv);
          END;|
  3    :  FOR count := 1 TO 40 DO
           FOR count2 := 1 TO 480 DO
            FOR plane := 1 TO 4 DO
             wplane(plane);
             screen1[count2,count]
             := quelle[plane]^[count2,count];
             screen1[count2,81 - count]
             := quelle[plane]^[count2,81 - count];
            END;
           END;
           Delay(speedv);
          END;|
  4    :  FOR count := 1 TO 240 DO
           FOR plane := 1 TO 4 DO
            wplane(plane);
            WordMove(ADR(quelle[plane]^[241 - count,1]),
                     ADR(screen1[241 - count,1]),40);
            WordMove(ADR(quelle[plane]^[240 + count,1]),
                     ADR(screen1[240 + count,1]),40);
           END;
           Delay(speedh);
          END;|
  5    :  FOR count := 1 TO 240 DO
           FOR plane := 1 TO 4 DO
            wplane(plane);
            WordMove(ADR(quelle[plane]^[count,1]),
                     ADR(screen1[count,1]),40);
            WordMove(ADR(quelle[plane]^[481 - count,1]),
                     ADR(screen1[481 - count,1]),40);
           END;
           Delay(speedh);
          END;|
  6    :
          FOR plane := 1 TO 4 DO
           wplane(plane);
           FOR count := 1 TO 480 DO
            WordMove(ADR(quelle[plane]^[count,1]),
                     ADR(screen1[count,1]),40);
            Delay(speedh);
           END;
          END;
 END;
END ToScreen16;

(* ---------------------------------- *)

PROCEDURE ToScreen256(quelle : virt256_pointer_typ;
                      mode : CARDINAL);
VAR  count  : CARDINAL;
     count2 : CARDINAL;
     count3 : CARDINAL;
BEGIN
 CASE mode OF
  1    :  Off();
          WordMove(quelle,ADR(screen2[1,1]),32000);
          On();|
  2    :  FOR count := 1 TO 160 DO
           FOR count2 := 1 TO 200 DO
             screen2[count2,160 + count]
             := quelle^[count2,160 + count];
             screen2[count2,161 - count]
             := quelle^[count2,161 - count];
           END;
           Delay(speedv);
          END;|
  3    :  FOR count := 1 TO 160 DO
           FOR count2 := 1 TO 200 DO
             screen2[count2,count] := quelle^[count2,count];
             screen2[count2,321 - count]
             := quelle^[count2,321 - count];
           END;
           Delay(speedv);
          END;|
  4    :  FOR count := 1 TO 100 DO
           WordMove(ADR(quelle^[101 - count,1]),
                    ADR(screen2[101 - count,1]),160);
           WordMove(ADR(quelle^[100 + count,1]),
                    ADR(screen2[100 + count,1]),160);
           Delay(speedh);
          END;|
  5    :  FOR count := 1 TO 100 DO
           WordMove(ADR(quelle^[count,1]),
                    ADR(screen2[count,1]),160);
           WordMove(ADR(quelle^[201 - count,1]),
                    ADR(screen2[201 - count,1]),160);
           Delay(speedh);
          END;|
  6    :  count  := 0;
          count2 := 40;
          FOR count3 := 1 TO count2 DO
           LOOP
            Move(AddAddr(quelle,count),
                 AddAddr(ADR(screen2[1,1]),count),1);
            IF count>64000-count2 THEN
             EXIT;
            ELSE
             INC(count,count2);
            END;
           END;
            count := count3;
          END;
 END;
END ToScreen256;

(********************************************)

(* Sucht den Anfang der Grafik-Daten *)
(* Return : Bei einem Fehler 0       *)
(* sonst die Lnge des BODY          *)
PROCEDURE SearchBody(datei:File):LONGCARD;
TYPE  string4            = ARRAY[1..4] OF CHAR;
VAR   result             : CARDINAL;
      ziel               : string4;
      kennung            : string4;
      dummy              : ARRAY[0..10000] OF CHAR;
      pos                : CARDINAL;
      lang               : LONGCARD;
BEGIN
 Seek(datei,0);
 result  := RdBin(datei,dummy,10000);
 pos     := 0;
 kennung := 'BODY';
 LOOP
  WordMove(ADR(dummy[pos]),ADR(ziel),2);
  IF ziel = kennung THEN
   EXIT;
  END;
  INC(pos);
  IF pos > 10000 THEN
   RETURN(0);
  END;
 END;
 WordMove(ADR(dummy[pos+4]),ADR(lang),2);
 lswap(lang);
 INC(pos,8);
 Seek(datei,LONGCARD(pos));
 RETURN (lang);
END SearchBody;

(********************************************)

PROCEDURE ReadHeader(datei:File):header_typ;
TYPE  string4            = ARRAY[1..4] OF CHAR;
VAR   result             : INTEGER;
      header             : header_typ;
      kennung            : string4;
      datkennung         : string4;
BEGIN
 kennung:='BMHD';
 Seek(datei,0CH);
 result := RdBin(datei,datkennung,4);
 IF kennung<>datkennung THEN
  header.typ := 'XXXX';
 END;
 Seek(datei,4);
 result := RdBin(datei,header.size,4);
 result := RdBin(datei,header.typ,4);
 Seek(datei,010H);
 result := RdBin(datei,header,24);
 WITH header DO
  lswap(lang);
  cswap(breit);
  cswap(hoch);
  cswap(transp);
  iswap(xpos);
  iswap(ypos);
  iswap(scbreit);
  iswap(schoch);
  lswap(size);
  INC(size,8);
 END;
 header.graphik := SearchBody(datei);
 RETURN (header);
END ReadHeader;

(********************************************)

PROCEDURE ReadPuffer(datei : File;lang : LONGCARD):puf_pointer_typ;
CONST   MAX   =   65000;
VAR     ort   :   puf_pointer_typ;
        mort  :   puf_pointer_typ;
        result:   CARDINAL;
        ptc   :   HeapRecPtr;
        lmem  :   LONGCARD;
BEGIN
 IF (HeapAvail(MainHeap)<CARDINAL(lang DIV 16 + 2)) THEN
  RETURN(NIL);
 END;
 HeapAllocate(MainHeap,ort,CARDINAL(lang DIV 16 + 2));
 IF lang<=MAX THEN                        (* BODY < 64k *)
  result := RdBin(datei,ort^,CARDINAL(lang));
 ELSE                                     (* BODY > 64k *)
  lmem := lang;
  mort := ort;
  result := RdBin(datei,mort^,CARDINAL(MAX));
  DEC(lmem,MAX);
  IncAddr(mort,MAX);
  LOOP
   IF lmem>MAX THEN
    result := RdBin(datei,mort^,CARDINAL(MAX));
    lmem   := lmem-MAX;
    IncAddr(mort,MAX);
   ELSE
    result := RdBin(datei,mort^,CARDINAL(lmem));
    EXIT;
   END;
  END;
 END;
 RETURN (ort);
END ReadPuffer;

(********************************************)

PROCEDURE ReadPalette(datei:File;pal : pal_pointer_typ;
                      anz : CARDINAL):BOOLEAN;
TYPE string4    = ARRAY[1..4] OF CHAR;
VAR  result     : INTEGER;
     lang       : LONGCARD;
     count      : INTEGER;
     kennung    : string4;
     datkennung : string4;
BEGIN
 Seek(datei,28H);
 kennung:='CMAP';
 result := RdBin(datei,datkennung,4);
 IF kennung<>datkennung THEN
  RETURN(FALSE);
 END;
 Seek(datei,30H);
 result := RdBin(datei,pal^,anz*3);
 FOR count := 1 TO anz DO
  WITH pal^[count] DO
   rt:=BYTE(SHORTCARD(rt)>>2);
   bl:=BYTE(SHORTCARD(bl)>>2);
   gr:=BYTE(SHORTCARD(gr)>>2);
  END;
 END;
 RETURN(TRUE);
END ReadPalette;

(********************************************)

PROCEDURE ReadLbm16(datei : File;header : header_typ;
                            puffer    : BOOLEAN;
                            ziel : virt16_pointer_typ)
                            : CARDINAL;
TYPE string4     = ARRAY[1..4] OF CHAR;
VAR  breit,hoch  : CARDINAL;
     planes      : CARDINAL;
     gelesen     : SHORTCARD;
     x,y,p,n,z   : CARDINAL;
     result      : CARDINAL;
     scx         : CARDINAL;
     kennung     : string4;
     datkennung  : string4;
     lang        : LONGCARD;
     ort         : puf_pointer_typ;
     memort      : puf_pointer_typ;
     dummy       : ARRAY[1..160] OF BYTE;
BEGIN
 IF header.pack = 0 THEN      (* Bei ungepackten Bilder    *)
  puffer := FALSE;            (* bringt puffern keinen     *)
 END;                         (* Zeitvorteil               *)
 x:=1;
 y:=1;
 p:=1;
 scx   := MAX18X+1;
 lang  := SearchBody(datei);  (* Anfang des Bildes suchen  *)
 IF lang = 0 THEN             (* BODY nicht gefunden       *)
  RETURN(1);
 END;
 breit := header.breit DIV 8; (* Anzahl der Byte pro Zeile *)
 hoch  := header.hoch+1;      (* Anzahl der Bildzeilen     *)
 IF header.breit MOD 8 <> 0 THEN
  INC(breit,1);               (* Justieren auf Bytegrenze  *)
 END;
 planes:= 4;
 IF header.maske=1 THEN       (* Sorry, Bilder mit Maske   *)
  RETURN(2)                   (* werden nicht gelesen      *)
 END;
 IF puffer THEN               (* Puffer anlegen            *)
  ort := ReadPuffer(datei,lang);
  IF ort = NIL THEN
   puffer := FALSE;
  ELSE
   memort := ort;
  END;
 END;
 IF header.pack = 1 THEN      (* Komprimiertes Bild einlesen*)
  REPEAT
   REPEAT
    wplane(p);                (* Bit Plane auswhlen *)
    REPEAT
     IF puffer THEN           (* Aus dem Puffer lesen *)
      gelesen := SHORTCARD(ort^);
      IncAddr(ort,1);
     ELSE                     (* Aus der Datei lesen  *)
      result:=RdBin(datei,gelesen,1);
     END;
     CASE gelesen OF
          0..127  : n:=CARDINAL(gelesen)+1;  (* unkomprimierte Bytes*)
                    IF puffer THEN (* Aus dem Puffer lesen *)
                      IF (x+n) < scx THEN
                      IF ziel=NIL THEN
                       (* Direkt in den Bildschirm *)
                       Move(ort,ADR(screen1[y,x]),n);
                      ELSE
                       (* In den virtuellen Schirm *)
                       Move(ort,ADR(ziel^[p]^[y,x]),n);
                      END;
                      ELSE
                       IF x<scx THEN
                        IF ziel = NIL THEN
                         (* Direkt in den Bildschirm *)
                         Move(ort,ADR(screen1[y,x]),scx-x);
                        ELSE
                         (* In den virtuellen Schirm *)
                         Move(ort,ADR(ziel^[p]^[y,x]),scx-x);
                        END;
                       END;
                      END;
                      IncAddr(ort,n);
                    ELSE  (* Aus der Datei lesen  *)
                     result := RdBin(datei,dummy,n);
                    IF x<scx THEN
                     IF ziel=NIL THEN
                      (* Direkt in den Bildschirm *)
                      Move(ADR(dummy),ADR(screen1[y,x]),n);
                     ELSE
                      (* Aus dem Puffer lesen *)
                      Move(ADR(dummy),ADR(ziel^[p]^[y,x]),n);
                     END;
                    END;
                    END;
                    INC(x,n);

          |129..256:n:=257 -CARDINAL(gelesen); (* komprimierte Bytes*)
                    IF puffer THEN  (* Aus dem Puffer lesen *)
                     gelesen := SHORTCARD(ort^);
                     IncAddr(ort,1);
                    ELSE            (* Aus der Datei lesen  *)
                     result:=RdBin(datei,gelesen,1);
                    END;
                    z:=1;
                    REPEAT
                     IF x<scx THEN
                      IF ziel=NIL THEN
                       (* Direkt in den Bildschirm *)
                       screen1[y,x]:=gelesen;
                      ELSE
                       (* In den virtuellen Schirm *)
                       ziel^[p]^[y,x]:= gelesen;
                      END;
                     END;
                     INC(x);
                     INC(z);
                    UNTIL z>n;
     END;
    UNTIL (x>breit);
   x:=1;
   p:=p+1;
   UNTIL (p>planes);
   p:=1;
   x:=1;
   y:=y+1;
  UNTIL (y>=hoch) OR (y>MAX18Y);
 ELSE                        (* Unkomprimiertes Bild einlesen*)
  REPEAT
   FOR p := 1 TO 4 DO
    wplane(p);               (* Bit Plane auswhlen *)
    IF ziel=NIL THEN
     (* Direkt in den Bildschirm *)
     result := RdBin(datei,screen1[y,1],breit);
    ELSE
     (* In den virtuellen Schirm *)
     result := RdBin(datei,ziel^[p]^[y,x],breit);
    END;
   END;
   INC(y);
  UNTIL (y>=hoch) OR (y>MAX18Y);
 END;
 IF puffer THEN              (* Puffer wieder freigeben *)
  HeapDeallocate(MainHeap,memort,CARDINAL(lang DIV 16 + 2));
 END;
 RETURN(0);
END ReadLbm16;

(********************************************)

PROCEDURE ReadLbm256(datei : File;
                     header: header_typ;
                     puffer: BOOLEAN;
                     ziel  : virt256_pointer_typ)
                     : CARDINAL;
TYPE string4     = ARRAY[1..4] OF CHAR;

VAR  breit,hoch  : CARDINAL;
     planes      : CARDINAL;
     gelesen     : SHORTCARD;
     x,y,p,n,z   : CARDINAL;
     result      : CARDINAL;
     scr         : CARDINAL;
     kennung     : string4;
     datkennung  : string4;
     lang        : LONGCARD;
     ort         : puf_pointer_typ;
     memort      : puf_pointer_typ;

BEGIN
 IF header.maske=1 THEN       (* Sorry, Bilder mit Maske   *)
  RETURN(2);                  (* werden nicht gelesen      *)
 END;
 x:=1;
 y:=1;
 p:=1;
 result := 0;
 scr    := MAX19X + 1;
 lang   := SearchBody(datei);   (* Anfang des Bildes suchen  *)
 IF lang = 0 THEN               (* BODY nicht gefunden       *)
  RETURN(1);
 END;
 breit  := header.breit;        (* Anzahl der Byte pro Zeile *)
 hoch   := header.hoch+1;       (* Anzahl der Bildzeilen     *)
 IF puffer THEN                 (* Puffer anlegen            *)
  ort := ReadPuffer(datei,lang);
  IF ort = NIL THEN
   puffer := FALSE;
  ELSE
   memort := ort;
  END;
 END;
  REPEAT
    REPEAT
     IF puffer THEN  (* Aus dem Puffer lesen *)
      gelesen := SHORTCARD(ort^);
      IncAddr(ort,1);
     ELSE            (* Aus der Datei lesen  *)
      result:=RdBin(datei,gelesen,1);
     END;
     CASE gelesen OF
          0..127  : n:=CARDINAL(gelesen)+1;  (* Unkomprimierte Bytes*)
                    FOR z:=1 TO n DO
                     IF puffer THEN (* Aus dem Puffer lesen *)
                      gelesen := SHORTCARD(ort^);
                      IncAddr(ort,1);
                     ELSE           (* Aus der Datei lesen  *)
                      result:=RdBin(datei,gelesen,1);
                     END;
                     IF (x<MAX19X) THEN
                      IF ziel=NIL THEN
                       (* Direkt in den Bildschirm *)
                       screen2[y,x]:=gelesen;
                      ELSE
                       (* In den virtuellen Schirm *)
                       ziel^[y,x]:=gelesen;
                      END;
                     END;
                     INC(x);
                    END;

          |129..256:n:=257 -CARDINAL(gelesen); (* Komprimierte Bytes*)
                    IF puffer THEN  (* Aus dem Puffer lesen *)
                     gelesen := SHORTCARD(ort^);
                     IncAddr(ort,1);
                    ELSE            (* Aus der Datei lesen  *)
                     result:=RdBin(datei,gelesen,1);
                    END;
                    FOR z:=1 TO n DO
                     IF (x<scr) THEN
                      IF ziel=NIL THEN
                       (* Direkt in den Bildschirm *)
                       screen2[y,x]:=gelesen;
                      ELSE
                       (* In den virtuellen Schirm *)
                       ziel^[y,x]:=gelesen;
                      END;
                     END;
                     INC(x);
                    END;
           ELSE
     END;
    UNTIL (x>breit);
   x:=1;
   y:=y+1;
  UNTIL (y>=hoch) OR (y>MAX19Y);
  RETURN(0);
END ReadLbm256;

(********************************************)

PROCEDURE ReadLbmAlt256(datei : File;
                        header : header_typ;
                        puffer: BOOLEAN;
                        ziel  : virt256_pointer_typ)
                        : CARDINAL;
TYPE string4     = ARRAY[1..4] OF CHAR;
VAR  breit,hoch  : CARDINAL;
     planes      : CARDINAL;
     gelesen     : SHORTCARD;
     x,y,p,n,z   : CARDINAL;
     p2,n2,z2    : CARDINAL;
     result      : CARDINAL;
     scr         : CARDINAL;
     kennung     : string4;
     datkennung  : string4;
     lang        : LONGCARD;
     ort         : puf_pointer_typ;
     memort      : puf_pointer_typ;
     ds          : ARRAY[1..100] OF BYTE;
     chk         : BOOLEAN;

BEGIN
 IF header.maske=1 THEN       (* Sorry, Bilder mit Maske   *)
  RETURN(2);                  (* werden nicht gelesen      *)
 END;
 x:=1;
 y:=1;
 p:=1;
 result := 0;
 scr    := MAX19X + 1;
 lang   := SearchBody(datei);   (* Anfang des Bildes suchen  *)
 IF lang = 0 THEN               (* BODY nicht gefunden       *)
  RETURN(1);
 END;
 breit  := header.breit         (* Anzahl der Byte pro Zeile *)
           DIV 8 + 1;
 IF header.breit MOD 8 <> 0 THEN
  INC(breit,1);                 (* Justieren auf Bytegrenze  *)
 END;
 hoch   := header.hoch+1;       (* Anzahl der Bildzeilen     *)
 IF header.pack = 0 THEN        (* Bei ungepackten Bilder    *)
  puffer := FALSE;              (* bringt puffern keinen     *)
 END;                           (* Zeitvorteil               *)
 IF puffer THEN                 (* Puffer anlegen            *)
  ort := ReadPuffer(datei,lang);
  IF ort = NIL THEN
   puffer := FALSE;
  ELSE
   memort := ort;
  END;
 END;
 IF header.pack = 1 THEN        (* Komprimiertes Bild einlesen*)
  REPEAT
   FOR p := 0 TO 7 DO
    REPEAT
     IF puffer THEN      (* Aus dem Puffer lesen *)
      gelesen := SHORTCARD(ort^);
      IncAddr(ort,1);
     ELSE                (* Aus der Datei lesen  *)
      result:=RdBin(datei,gelesen,1);
     END;
     CASE gelesen OF
          0..127  : n:=CARDINAL(gelesen)+1;  (* Unkomprimierte Bytes*)
                     IF puffer THEN  (* Aus dem Puffer lesen *)
                      Move(ort,ADR(ds[x]),n);
                      IncAddr(ort,n);
                     ELSE            (* Aus der Datei lesen  *)
                      result:=RdBin(datei,ds[x],n);
                     END;
                     INC(x,n);
          |129..256:n:=257 -CARDINAL(gelesen); (* Komprimierte Bytes*)
                    IF puffer THEN   (* Aus dem Puffer lesen *)
                     gelesen := SHORTCARD(ort^);
                     IncAddr(ort,1);
                    ELSE             (* Aus der Datei lesen  *)
                     result:=RdBin(datei,gelesen,1);
                    END;
                    Fill(ADR(ds[x]),n,gelesen);
                    INC(x,n);

    END;
   UNTIL (x>=breit);
   FOR p2 := 1 TO breit-1 DO
    FOR z2 := 1 TO 8 DO
     chk := 8-z2 IN BITSET(ds[p2]);
     IF chk THEN
      IF ziel=NIL THEN
       (* Direkt in den Bildschirm *)
       INCL(BITSET(screen2[y,(p2-1)*8+z2]),p);
      ELSE
       (* In den virtuellen Schirm *)
       INCL(BITSET(ziel^[y,(p2-1)*8+z2]),p);
      END;
     END;
    END;
   END;
   x:=1;
   END;
   y:=y+1;
  UNTIL (y>=hoch) OR (y>MAX19Y);
 ELSE   (*  Unkomprimiertes Bild einlesen  *)
  y:=1;
  REPEAT
   FOR n2 := 1 TO 8 DO
    result := RdBin(datei,ds,40);
    FOR p2 := 1 TO 40 DO
     FOR z2 := 1 TO 8 DO
      chk := 8-z2 IN BITSET(ds[p2]);
      IF chk THEN
       IF ziel=NIL THEN
        (* Direkt in den Bildschirm *)
        INCL(BITSET(screen2[y,(p2-1)*8+z2]),n2-1);
       ELSE
        (* In den virtuellen Schirm *)
        INCL(BITSET(ziel^[y,(p2-1)*8+z2]),n2-1);
       END;
      END;
     END;
    END;
   END;
  INC(y,1);
  UNTIL (y>=hoch) OR (y>MAX19Y);
 END;
 IF puffer THEN               (* Puffer freigeben *)
  HeapDeallocate(MainHeap,memort,CARDINAL(lang DIV 16 + 2));
 END;
 RETURN(0);
END ReadLbmAlt256;

(********************************************)

PROCEDURE SetColors(datei : File;anz : CARDINAL):BOOLEAN;
VAR    pal   : pal256_typ;
       pap   : pal_pointer_typ;
       regi  : Registers;
BEGIN
 pap := ADR(pal);
 IF NOT ReadPalette(datei,pap,anz) THEN
  RETURN(FALSE);
 END;
 WITH regi DO
  AH := 010H;
  AL := 012H;
  BX := 0;
  CX := anz;
  ES := Seg(pal);
  DX := Ofs(pal);
  Wait();
  Intr(regi,010H);
 END;
 RETURN(TRUE);
END SetColors;

(********************************************)

PROCEDURE SetColorsPap(pal : pal_pointer_typ; anz : CARDINAL);
VAR  regi   : Registers;
BEGIN
 WITH regi DO
  AH := 010H;
  AL := 012H;
  BX := 0;
  CX := anz;
  ES := Seg(pal^);
  DX := Ofs(pal^);
  Wait();
  Intr(regi,010H);
 END;
END SetColorsPap;

(********************************************)

PROCEDURE LeseBild(name     : ARRAY OF CHAR;
                   puffer   : BOOLEAN;
                   auto     : BOOLEAN;
                   farb     : BOOLEAN;
                   bild     : BOOLEAN;
                   ziel     : BOOLEAN;
                   mode     : CARDINAL)
                   :CARDINAL;
VAR   datei    :  File;
      header   :  header_typ;
      farben   :  CARDINAL;
      chk      :  BOOLEAN;
      zp16     :  virt16_typ;
      zp16po   :  virt16_pointer_typ;
      zp256po  :  virt256_pointer_typ;

BEGIN
 datei    := Open(name);
 (* Prfen ob berhaupt eine IFF Datei *)
 IF NOT IstLbm(datei) THEN
  RETURN(1);
 END;
 header   := ReadHeader(datei);
 WITH header DO
  IF header.typ[1] = 'X' THEN
   RETURN(2);
  END;
  CASE tiefe OF
   4   : IF auto THEN
          IF NOT SetVideoMode(18) THEN
           RETURN(3);
          END;
         END;
         zp16po := NIL;
         IF ziel THEN
          zp16:=MakeVirt16();
          IF zp16[1]=NIL THEN
           ziel := FALSE;
          ELSE
           zp16po := ADR(zp16);
          END;
         END;
         IF farb THEN
          chk := SetColors(datei,16);
         END;
         IF bild THEN
          IF ReadLbm16(datei,header,puffer,zp16po)
          <>0 THEN
           RETURN(4);
          END;
         END;
         IF zp16po<> NIL THEN
          ToScreen16(zp16,mode);
          RemoveVirt16(zp16);
         END;|
   8   : IF auto THEN
          IF NOT SetVideoMode(19) THEN
           RETURN(3);
          END;
         END;
         zp256po := NIL;
         IF ziel THEN
          zp256po := MakeVirt256();
         END;
         IF farb THEN
          chk := SetColors(datei,256);
         END;
         IF bild THEN
          IF (header.typ[1]='P') AND
             (header.pack=1)     THEN
           IF ReadLbm256(datei,header,puffer,zp256po)
           <>0 THEN
            RETURN(4);
           END;
          ELSE
           IF ReadLbmAlt256(datei,header,puffer,zp256po)
           <>0 THEN
            RETURN(4);
           END;
          END;
         END;
         IF zp256po<>NIL THEN
          ToScreen256(zp256po,mode);
          RemoveVirt256(zp256po);
         END;
  END;
 END;
 Close(datei);
 RETURN(0);
END LeseBild;

(********************************************)

BEGIN
 speedh := 10;
 speedv := 0;
END lbm.
